/* 
* cmh_menus.c
*
* Collection of all menus for Chicken Mobile Home
*/

#ifndef cmh_f_cpu_
#define cmh_f_cpu_

#define F_CPU 8000000L

#endif

#include <inttypes.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "chicken_mobile.h"
#include "i2cmaster.h"
#include "DS18B20_routines.h"
#include "glcd_routines.h"
#include "keyboard_routines.h"
#include "display_routines.h"
#include "cmh_util.h"
#include "ee_routines.h"

/* edit_date_time ------------------------------------------------------------------------*
* Uses the keyboard to edit date and time
* Returns: 0 - okay
*        > 0 - cancel, escape
*/
uint8_t edit_date_time(void) {
	extern datetime_t date_time;
	extern uint8_t sec_counter;
	datetime_t my_date_time;
	extern const char str_edit_year[];
	extern const char str_edit_month[];
	extern const char str_edit_day[];
	extern const char str_edit_hour[];
	extern const char str_edit_minute[];
	extern const char msg_datetime_set[];
	extern const char msg_cancelled[];
	
	edit_frame_t ef_dt;
	ef_dt.value = date_time.year;
	ef_dt.string = str_edit_year;
	ef_dt.min = 15;
	ef_dt.max = 25;
	ef_dt.step = 0;
	ef_dt.decimal_places = 0;
	if (edit_data_item(&ef_dt) == 0) {
		my_date_time.year = (uint8_t) ef_dt.value;
		ef_dt.value = date_time.month;
		ef_dt.string = str_edit_month;
		ef_dt.min = 1;
		ef_dt.max = 12;
		if (edit_data_item(&ef_dt) == 0) {
			my_date_time.month = (uint8_t) ef_dt.value;
			ef_dt.value = date_time.day;
			ef_dt.string = str_edit_day;
			ef_dt.min = 1;
			ef_dt.max = 31;
			if (edit_data_item(&ef_dt) == 0) {
				my_date_time.day = (uint8_t) ef_dt.value;
				ef_dt.value = date_time.hour;
				ef_dt.string = str_edit_hour;
				ef_dt.min = 1;
				ef_dt.max = 23;
				if (edit_data_item(&ef_dt) == 0) {
					my_date_time.hour = (uint8_t) ef_dt.value;
					ef_dt.value = date_time.min;
					ef_dt.string = str_edit_minute;
					ef_dt.min = 1;
					ef_dt.max = 59;
					if (edit_data_item(&ef_dt) == 0) {
						my_date_time.min = (uint8_t) ef_dt.value;
						// set date and time
						i2c_start_wait(2 * RTC_ADDR + I2C_WRITE);
						i2c_write(0);
						i2c_write(0);		// seconds
						i2c_write(hex_to_bcd(my_date_time.min));
						i2c_write(hex_to_bcd(my_date_time.hour));
						i2c_write(0);		// day of week - not used
						i2c_write(hex_to_bcd(my_date_time.day));
						i2c_write(hex_to_bcd(my_date_time.month));
						i2c_write(hex_to_bcd(my_date_time.year));
						i2c_stop();
						date_time = my_date_time;
						sec_counter = 0;
						message_box(msg_datetime_set, 2);
						return(0);
					};
				};
			};
		};
	};
	message_box(msg_cancelled, 2);
	return(1);					// operation aborted
};

/* edit display parameters ------------------------------------------------------------------*/
void edit_display_parameters(void) {
	extern display_status_t display_status;
	extern const char msg_display_set[];
	extern const char msg_cancelled[];
	extern const char str_edit_timeout[];
	extern const char str_edit_backlight[];
	edit_frame_t edit_display_status;
	uint8_t display_light_level;
	
	// edit light level
	edit_display_status.value = display_status.light_level;
	edit_display_status.string = str_edit_backlight;
	edit_display_status.min = 0;
	edit_display_status.max = MAX_DISPLAY_LIGHT;
	edit_display_status.step = 0;
	edit_display_status.decimal_places = 0;
	if (edit_data_item(&edit_display_status) == 0) {
		display_light_level = (uint8_t) edit_display_status.value;		// buffer new light level
		// edit timeout
		edit_display_status.value = display_status.timeout / 60;
		edit_display_status.string = str_edit_timeout;
		edit_display_status.min = 1;
		edit_display_status.max = 30;
		if (edit_data_item(&edit_display_status) == 0) {
			display_status.light_level = display_light_level;			// set new light level
			display_status.timeout = edit_display_status.value * 60;	// set new time out
			ee_write_display_status();									// save data to eeprom
			set_display_light(display_status.light_level);				// adjust light level 
			message_box(msg_display_set, 2);							// finish  editing
			return;
		};
	};
	message_box(msg_cancelled, 2);
}

/* edit_LED_time_window_parms -----------------------------------------------------------------*/
void edit_led_tw_parms(void) {
	extern LED_status_t LED_status;
	extern const char msg_led_tw_set[];
	extern const char msg_cancelled[];
	extern const char str_edit_led_tw_on_hour[];
	extern const char str_edit_led_tw_on_min[];
	extern const char str_edit_led_tw_off_hour[];
	extern const char str_edit_led_tw_off_min[];
	edit_frame_t edit_frame;
	uint8_t buffer_tw_on_hour, buffer_tw_on_min, buffer_tw_off_hour;
	
	// edit tw on hour
	edit_frame.value = LED_status.led_tw_on_hour;
	if (edit_frame.value < 12) edit_frame.value = 12;
	if (edit_frame.value > 21) edit_frame.value = 21;
	edit_frame.string = str_edit_led_tw_on_hour;
	edit_frame.min = 12;
	edit_frame.max = 21;
	edit_frame.step = 5;
	edit_frame.decimal_places = 0;
	if (edit_data_item(&edit_frame) == 0) {
		buffer_tw_on_hour = (uint8_t) edit_frame.value;				
		// edit tw on min
		edit_frame.value = LED_status.led_tw_on_min;
		edit_frame.string = str_edit_led_tw_on_min;
		edit_frame.min = 0;
		edit_frame.max = 59;
		edit_frame.step = 10;
		if (edit_data_item(&edit_frame) == 0) {
			buffer_tw_on_min = (uint8_t) edit_frame.value;			
			// edit tw off hour
			edit_frame.value = LED_status.led_tw_off_hour;
			if (edit_frame.value < buffer_tw_on_hour + 1) edit_frame.value = buffer_tw_on_hour + 1;
			edit_frame.string = str_edit_led_tw_off_hour;
			edit_frame.min = buffer_tw_on_hour + 1;
			edit_frame.max = 23;
			edit_frame.step = 5;
			if (edit_data_item(&edit_frame) == 0) {
				buffer_tw_off_hour = (uint8_t) edit_frame.value;
				// edit tw on min
				edit_frame.value = LED_status.led_tw_off_min;
				edit_frame.string = str_edit_led_tw_off_min;
				edit_frame.min = 0;
				edit_frame.max = 59;
				edit_frame.step = 10;
				if (edit_data_item(&edit_frame) == 0) {
					LED_status.led_tw_on_hour = buffer_tw_on_hour;
					LED_status.led_tw_on_min = buffer_tw_on_min;
					LED_status.led_tw_off_hour = buffer_tw_off_hour;	
					LED_status.led_tw_off_min = (uint8_t) edit_frame.value;	
					ee_write_LED_status();				// save data to eeprom
					message_box(msg_led_tw_set, 2);		// finish  editing
					return;
				};
			};
		};
	};
	message_box(msg_cancelled, 2);
}

/* edit_LED_dur_max_parms -----------------------------------------------------------------*/
void edit_led_dur_max_parms(void) {
	extern LED_status_t LED_status;
	extern const char msg_led_dur_max_set[];
	extern const char msg_cancelled[];
	extern const char str_edit_led_dur[];
	extern const char str_edit_led_max[];
	edit_frame_t edit_frame;
	uint8_t buffer_dur;
	
	// edit duration
	edit_frame.value = LED_status.led_duration;
	edit_frame.string = str_edit_led_dur;
	edit_frame.min = 30;
	edit_frame.max = 240;
	edit_frame.step = 30;
	edit_frame.decimal_places = 0;
	if (edit_data_item(&edit_frame) == 0) {
		buffer_dur = (uint8_t) edit_frame.value;
		// edit maximum
		edit_frame.value = LED_status.led_max;
		edit_frame.string = str_edit_led_max;
		edit_frame.min = 5;
		edit_frame.max = 30;
		edit_frame.step = 5;
		if (edit_data_item(&edit_frame) == 0) {
			LED_status.led_duration = buffer_dur;
			LED_status.led_max = (uint8_t) edit_frame.value;
			ee_write_LED_status();						// save data to eeprom
			message_box(msg_led_dur_max_set, 2);		// finish  editing
			return;
		};
	};
	message_box(msg_cancelled, 2);
}

/* edit_amb_light_parms-----------------------------------------------------------------*/
void edit_amb_light_parms(void) {
	extern amb_light_status_t amb_light_status;
	extern const char msg_amb_light_set[];
	extern const char msg_cancelled[];
	extern const char str_edit_amb_light_thr[];
	extern const char str_edit_amb_light_dur[];
	edit_frame_t edit_frame;
	uint16_t buffer_thr;
	
	// edit threshold
	edit_frame.value = amb_light_status.light_threshold;
	edit_frame.string = str_edit_amb_light_thr;
	edit_frame.min = 100;
	edit_frame.max = 900;
	edit_frame.step = 50;
	edit_frame.decimal_places = 0;
	if (edit_data_item(&edit_frame) == 0) {
		buffer_thr = edit_frame.value;
		// edit duration
		edit_frame.value = amb_light_status.light_mov_avg_dur;
		edit_frame.string = str_edit_amb_light_dur;
		edit_frame.min = 2;
		edit_frame.max = MAX_AMB_LIGHT_MOV_AVG_DUR;
		edit_frame.step = 0;
		if (edit_data_item(&edit_frame) == 0) {
			amb_light_status.light_threshold = buffer_thr;
			amb_light_status.light_mov_avg_dur = (uint8_t) edit_frame.value;
			ee_write_amb_light_status();			// save data to eeprom
			message_box(msg_amb_light_set, 2);		// finish  editing
			return;
		};
	};
	message_box(msg_cancelled, 2);
}

/* edit hatch control parms -----------------------------------------------------------------*/
void edit_hatch_control_parms(void) {
	extern hatch_control_t hatch_control;
	extern const char msg_hatch_control_set[];
	extern const char msg_cancelled[];
	extern const char str_edit_hatch_control_hour[];
	extern const char str_edit_hatch_control_min[];
	extern const char str_edit_hatch_control_dur[];
	edit_frame_t edit_frame;
	uint8_t buffer_hour, buffer_min;
	
	// edit hour 
	edit_frame.value = hatch_control.hatch_hour; 
	edit_frame.string = str_edit_hatch_control_hour;
	edit_frame.min = 0;
	edit_frame.max = 23;
	edit_frame.step = 5;
	edit_frame.decimal_places = 0;
	if (edit_data_item(&edit_frame) == 0) {
		buffer_hour = (uint8_t) edit_frame.value;				// buffer new hour
		// edit min
		edit_frame.value = hatch_control.hatch_min;
		edit_frame.string = str_edit_hatch_control_min;
		edit_frame.min = 0;
		edit_frame.max = 59;
		edit_frame.step = 10;
		if (edit_data_item(&edit_frame) == 0) {
			buffer_min = (uint8_t) edit_frame.value;			// buffer new min
			// edit duration	
			edit_frame.value = hatch_control.hatch_dur;
			edit_frame.string = str_edit_hatch_control_dur;
			edit_frame.min = 5;
			edit_frame.max = 900;
			edit_frame.step = 30;
			if (edit_data_item(&edit_frame) == 0) {
				hatch_control.hatch_hour = buffer_hour;
				hatch_control.hatch_min = buffer_min;
				hatch_control.hatch_dur = edit_frame.value;
				ee_write_hatch_control();						// save data to eeprom
				message_box(msg_hatch_control_set, 2);			// finish  editing
				return;
			};
		}; 
	};
	message_box(msg_cancelled, 2);
}

/* ---------------------------------------------------------------------------------------*/
void run_test_output(void) {
	extern display_status_t display_status;
	extern LED_status_t LED_status;
	extern mp3_status_t mp3_status;
	extern hatch_control_t hatch_control;
	uint8_t keyboard_value, stop = 0, LED_level_buffer, mp3_song_nr;
	extern const char str_menu_test_output[];
	extern uint8_t job_flags;
	
	const uint8_t x_pos1 = 0, x_pos2 = 70, x_pos3 = 140, y_pos = GLCD_WORKSPACE_Y_START + 10 + 9;
	
	LED_level_buffer = LED_status.led_level;
	LED_status.led_mode = LED_MAN;

	mp3_status.mp3_mode = MP3_MAN;
	switch_mp3_off();
	mp3_song_nr = mp3_status.song_nr;
	mp3_status.song_nr = 1;
	
	hatch_control.hatch_mode = HATCH_MAN;

	show_menu(str_menu_test_output);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	glcd_draw_line(x_pos2 - 10, GLCD_WORKSPACE_Y_START, x_pos2 - 10, GLCD_WORKSPACE_Y_START + 10 + 27, 1);
	glcd_draw_line(x_pos3 - 10, GLCD_WORKSPACE_Y_START, x_pos3 - 10, GLCD_WORKSPACE_Y_START + 10 + 27, 1);
	show_mp3_status(x_pos3, y_pos);
	show_mp3_song_nr(x_pos3, y_pos + 9);
	do {
		if (display_status.timeout_cnt == 0) keyboard_value = 1;	// in case of timeout: auto-cancel 
		else keyboard_value = read_keyboard();						// otherwise check whether key was pressed
		
		if (job_flags & (1 << JF_REFRESH_DISPLAY)) {
			job_flags &= ~(1 << JF_REFRESH_DISPLAY); 
			show_LED_level(x_pos1, y_pos);
			show_hatch_status(x_pos2, y_pos);
		};
		
		switch (keyboard_value) {
			case 1:			// return back
				stop = 1;
				break;
			case 2:			// switch hatch
				if (hatch_control.hatch_cnt == 0) {
					hatch_switch_on();
				} else {
					hatch_switch_off();
				};
				break;
			case 3:			// switch MP3 on/off
				if (mp3_status.mp3_status == MP3_OFF)
					switch_mp3_on();
				else
					switch_mp3_off();
				show_mp3_status(70, GLCD_WORKSPACE_Y_START + 10 + 18);
				break;
			case 4:			// switch MP3 song number 
				if (mp3_status.mp3_status == MP3_OFF) {
					if (mp3_status.song_nr < MAX_MP3_SONGS) 
						++mp3_status.song_nr;
					else
						mp3_status.song_nr = 1;
					show_mp3_song_nr(120, GLCD_WORKSPACE_Y_START + 10 + 18);
				};
				break;
			case 5:
				if (LED_status.led_level < MAX_LED_LEVEL) set_LED_level(LED_status.led_level + 1);
				break;
			case 6:
				if (LED_status.led_level > 0) set_LED_level(LED_status.led_level - 1);
			break;
		};
	} while (stop == 0);
	
	LED_status.led_mode = LED_AUTO;
	set_LED_level(LED_level_buffer);
	
	mp3_status.mp3_mode = MP3_AUTO;
	mp3_status.song_nr = mp3_song_nr;
	
	hatch_control.hatch_mode = HATCH_AUTO;
	
	return;
}

/* ---------------------------------------------------------------------------------------*/
static void show_setup_parms_1(void) {
	extern const char str_menu_setup_1[];
	const uint8_t pos2 = 80;

	show_menu(str_menu_setup_1);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	show_display_parms(0);
	glcd_draw_line(pos2, GLCD_WORKSPACE_Y_START, pos2, GLCD_WORKSPACE_Y_START + 35, 1);
	show_datetime_parms(pos2 + 4);
}

/* ---------------------------------------------------------------------------------------*/
static void show_setup_parms_2(void) {
	extern const char str_menu_setup_2[];
	const uint8_t pos2 = 74;

	show_menu(str_menu_setup_2);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	show_batt_parms(0);				// show battery parameters
	glcd_draw_line(pos2, GLCD_WORKSPACE_Y_START, pos2, GLCD_WORKSPACE_Y_START + 35, 1);
	show_LED_parms(pos2 + 4);		// show LED parameters
}

/* ---------------------------------------------------------------------------------------*/
static void show_setup_parms_3(void) {
	extern const char str_menu_setup_3[];
	const uint8_t pos2 = 56;

	show_menu(str_menu_setup_3);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	show_amb_light_parms(0);		// show ambient light sensor parameters
	glcd_draw_line(pos2, GLCD_WORKSPACE_Y_START, pos2, GLCD_WORKSPACE_Y_START + 35, 1);
	show_mp3_parms(pos2 + 4);		// show MP3 control parameters

}

/* ---------------------------------------------------------------------------------------*/
static void show_setup_parms_4(void) {
	extern const char str_menu_setup_4[];
	const uint8_t pos2 = 79;

	show_menu(str_menu_setup_4);
	glcd_draw_filled_rectangle(0, GLCD_WORKSPACE_Y_START, GLCD_MAXX - 1, GLCD_WORKSPACE_Y_END, 0);
	show_hatch_control_parms(0);
	glcd_draw_line(pos2, GLCD_WORKSPACE_Y_START, pos2, GLCD_WORKSPACE_Y_START + 35, 1);

}

/* ---------------------------------------------------------------------------------------*/
void run_setup_menu(void) {
	extern const char str_menu_setup_2[];
	extern display_status_t display_status;
	uint8_t keyboard_value, setup_mode = 1, stop = 0;
	
	show_setup_parms_1();
	
	do {
		if (display_status.timeout_cnt == 0) keyboard_value = 1;	// in case of timeout: auto-cancel
		else keyboard_value = read_keyboard();						// otherwise check whether key was pressed
		switch (keyboard_value) {
			case 1:					// cancel and return back
				stop = 1;
				break;
			case 2:
				switch (setup_mode) {
					case 3: edit_amb_light_parms();
							break;
					case 4: edit_hatch_control_parms();
							break;
				};
				stop = 1;
				break;
			case 3:					// edit display parameters
				switch (setup_mode) {
					case 1: edit_display_parameters();
							break;
					case 2: edit_led_tw_parms();
							break;
				};
				stop = 1;
				break;
			case 4:
				switch (setup_mode) {
					case 1: edit_date_time();
							break;
					case 2: edit_led_dur_max_parms();
							break;
				};
				stop = 1;
				break;
			case 5:
				if (++setup_mode > 4) setup_mode = 1;
				switch (setup_mode) {
					case 1: show_setup_parms_1();
							break;
					case 2: show_setup_parms_2();
							break;
					case 3: show_setup_parms_3();
							break;
					case 4: show_setup_parms_4();
				};
				break;
			case 6:
				if (--setup_mode == 0) setup_mode = 4;
				switch (setup_mode) {
					case 1: show_setup_parms_1();
							break;
					case 2: show_setup_parms_2();
							break;
					case 3: show_setup_parms_3();
							break;
					case 4: show_setup_parms_4();
				};
				break;
		};
	} while (stop == 0);
	return;
}

